(in-package :clim-user)

;; (defmethod initialize-instance :after ((pane radio-layout-pane)
;; 				       &rest args
;; 				       &key contents
;; 				       &allow-other-keys)
;;   (declare (ignore args))
;;   (let (in-cdr)
;;     (dolist (child contents)
;;       (if in-cdr
;;           (setf (sheet-enabled-p child) nil)
;;           (progn 
;;             (setf in-cdr t)
;;             (setf (enabled-radio-pane pane) child)))
;;       (sheet-adopt-child pane child))))

(defclass radio-layout-pane (stack-layout-pane)
  ((enabled-radio-pane :initform nil :accessor enabled-radio-pane)))

(defgeneric enabled-pane (parent))

(defmethod enabled-pane ((parent radio-layout-pane))
  (enabled-radio-pane parent))

(defmethod initialize-instance :after ((pane radio-layout-pane)
				       &rest args
				       &key contents
				       &allow-other-keys)
  (declare (ignore args))
  (setf (enabled-radio-pane pane) (car contents))
  (dolist (child (cdr contents))
    (setf (sheet-enabled-p child) nil)))

(defgeneric switch-to-pane (pane parent))

(defmethod switch-to-pane ((pane sheet) (parent radio-layout-pane))
  (unless (equal pane
                 (enabled-radio-pane parent))
    (setf (sheet-enabled-p (enabled-radio-pane parent))
          nil
          (sheet-enabled-p pane)
          t
          (enabled-radio-pane parent)
          pane)))

(defmethod switch-to-pane ((pane sheet) (parent (eql 'radio-layout-pane)))
  (switch-to-pane pane (sheet-parent pane)))

(defmethod add-pane :after ((pane sheet) (parent radio-layout-pane)  &optional enabled)
  (when enabled
    (switch-to-pane pane parent)))

(defmethod remove-pane :after ((pane sheet) (parent radio-layout-pane))
  (let ((last-of-children (car (last (sheet-children parent)))))
    (when (and last-of-children
               (equal pane
                      (enabled-radio-pane parent)))
      (switch-to-pane last-of-children parent))))

(defmethod remove-pane ((pane sheet) (parent (eql 'radio-layout-pane)))
  (remove-pane pane (sheet-parent pane)))


;;; example code

(define-application-frame ooz ()
  ()
  (:panes
   (foo (make-pane 'push-button :label "Foo"))
   (bar (make-pane 'push-button :label "Bar"))
   (baz (make-pane 'push-button :label "Baz"))
   (io :interactor))
  (:layouts
   (default
       (vertically ()
         (make-pane 'radio-layout-pane
                    :contents (list foo bar baz))
         io))))

(defun ooz ()
  (run-frame-top-level (make-application-frame 'ooz)))

(defmacro make-switch-command (pane-name)
  `(define-ooz-command (,(read-from-string (concatenate 'string "com-" (string pane-name)))
                         :name ,(nstring-downcase (string pane-name)))
       ()
     (let ((pane (find-pane-named *application-frame* ',pane-name)))
       (switch-to-pane pane 'radio-layout-pane))))

(make-switch-command foo)
(make-switch-command bar)
(make-switch-command baz)

(define-ooz-command (com-break :name t)
    ()
  (break))

(define-ooz-command (com-add-pane :name t)
    ()
  (let ((fm (frame-manager *application-frame*)))
    (with-look-and-feel-realization (fm *application-frame*)
      (let ((foo-pane (find-pane-named *application-frame* 'foo)))
        (add-pane (make-pane 'push-button :name 'brr :label "Brr")
                  (sheet-parent foo-pane) t))))
  (make-switch-command brr)
  nil)

(define-ooz-command (com-remove-pane :name t)
    ()
  (let ((pane (find-pane-named *application-frame* '|brr|)))
    ;; (remove-pane pane (sheet-parent pane))
    (remove-pane pane 'radio-layout-pane))
  (remove-command-from-command-table 'com-brr 'ooz))
